package io.tswf.groovy.bettergroovy.asm.view.exception

import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.asm.view.meta.ViewMethodMetadata
import org.objectweb.asm.tree.ClassNode

@CompileStatic
class MethodMappingException extends RuntimeException {
    ClassNode targetClass
    ViewMethodMetadata viewMethodMeta

    MethodMappingException(ClassNode targetClass, ViewMethodMetadata viewMethodMeta) {
        super(getExceptionMessage("", targetClass, viewMethodMeta))
        this.targetClass = targetClass
        this.viewMethodMeta = viewMethodMeta
    }

    MethodMappingException(String message, ClassNode targetClass, ViewMethodMetadata viewMethodMeta) {
        super(getExceptionMessage(message, targetClass, viewMethodMeta))
        this.targetClass = targetClass
        this.viewMethodMeta = viewMethodMeta
    }

    MethodMappingException(String message, Throwable cause, ClassNode targetClass, ViewMethodMetadata viewMethodMeta) {
        super(getExceptionMessage(message, targetClass, viewMethodMeta), cause)
        this.targetClass = targetClass
        this.viewMethodMeta = viewMethodMeta
    }

    MethodMappingException(Throwable cause, ClassNode targetClass, ViewMethodMetadata viewMethodMeta) {
        super(getExceptionMessage("", targetClass, viewMethodMeta), cause)
        this.targetClass = targetClass
        this.viewMethodMeta = viewMethodMeta
    }

    static String getExceptionMessage(String message, ClassNode targetClass, ViewMethodMetadata viewMethodMeta) {
        return "Exception occurred while mapping view method ${viewMethodMeta.viewMethod.javaFullName} from ${viewMethodMeta.view.viewClass.javaName} into ${targetClass.javaName}: ${message}"
    }
}
