package io.tswf.groovy.bettergroovy.asm.view.generation.conflict

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.groovy.bettergroovy.asm.view.meta.ViewConflictResolveStrategy

@Slf4j
@AutoService(IViewConflictResolveLogic)
@CompileStatic
class WarnViewConflictResolver implements IViewConflictResolveLogic {

    // Загружаем самую приоритетную стратегию для логирования (нужно для окружений, где может быть любой кастомный логгер)
    @Lazy
    IViewConflictResolveWarningProvider loggingProvider = ServiceLoader.load(IViewConflictResolveWarningProvider)
        .toList()
            .stream()
                .reverseSortedBy { it.priority }
                .findFirst().orElseThrow { new RuntimeException("There is no view logging strategies found!") }


    @Override
    ViewConflictResolveStrategy getEnumValue() {
        return ViewConflictResolveStrategy.WARNING
    }

    @Override
    void handleConflictResolve(ConflictResolveContext resolveContext) {
        loggingProvider.log("The method ${resolveContext.conflictingMethod.javaFullName} is already presented in ${resolveContext.targetClass.javaName}, skipping mapping from ${resolveContext.viewMethodMeta.view.viewClass.javaName}...")
    }
}

interface IViewConflictResolveWarningProvider {
    void log(String logMessage)
    default int getPriority() { 0 }
}

@Slf4j
@AutoService(IViewConflictResolveWarningProvider)
@CompileStatic
class Slf4jViewConflictResolveWarningProvider implements IViewConflictResolveWarningProvider{

    public static final int PRIORITY = -1

    @Override
    void log(String logMessage) {
        log.warn(logMessage)
    }
}
