package io.tswf.groovy.bettergroovy.asm.view.reading.mappers

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.asm.view.annotation.MethodInvoke
import io.tswf.groovy.bettergroovy.asm.view.exception.InvalidViewMethodException
import io.tswf.groovy.bettergroovy.asm.view.generation.mapper.MethodInvokeMapper
import io.tswf.groovy.bettergroovy.asm.view.meta.ViewMethodMetadata
import io.tswf.groovy.bettergroovy.asm.view.reading.IViewMethodMapperReader
import io.tswf.groovy.bettergroovy.asm.view.reading.ViewMethodMapperReadingContext
import io.tswf.groovy.bettergroovy.asm.view.reading.remap.ViewsRemapper
import io.tswf.groovy.bettergroovy.asm.view.util.MethodToInvokeResolveHelper
import org.objectweb.asm.commons.Method

@AutoService(IViewMethodMapperReader)
@CompileStatic
class MethodInvokeMapperReader implements IViewMethodMapperReader {

    public static final Collection<String> PREFIXES = [ "invoke", "call" ]
    public static final String METHOD_NAME_PARAMETER_NAME = "name"
    public static final String METHOD_DESC_PARAMETER_NAME = "desc"

    @Override
    void handleViewMethodReading(ViewMethodMapperReadingContext readingContext) {
        def viewMethod = readingContext.viewMethodMeta.viewMethod
        def methodInvokeAnnotation = viewMethod.findAnnotationByType(MethodInvoke)
        def isViewMethodMatchesPrefixes = PREFIXES.any { prefix -> viewMethod.name.startsWith(prefix) && viewMethod.name.length() > prefix.length() }
        def methodIsInvocation = methodInvokeAnnotation != null || isViewMethodMatchesPrefixes

        if (methodIsInvocation) {
            def rawMethodName = methodInvokeAnnotation?.findValueByName(METHOD_NAME_PARAMETER_NAME, String) ?: MethodToInvokeResolveHelper.resolveTargetMethodNameFromViewMethod(readingContext.viewMethodMeta, PREFIXES)

            def rawMethodDesc = methodInvokeAnnotation?.findValueByName(METHOD_DESC_PARAMETER_NAME, String) ?: viewMethod.desc

            if (rawMethodName.isNullOrEmpty()) {
                throw new InvalidViewMethodException("extracted target method name is empty or null", readingContext.viewMethodMeta)
            }

            if (rawMethodDesc.isNullOrEmpty()) {
                throw new InvalidViewMethodException("extracted target method descriptor is empty or null", readingContext.viewMethodMeta)
            }

            // Проверки типов параметров очень громоздкие, так что реализованы только в самом маппере

            def rawMethod = new Method(rawMethodName, rawMethodDesc)

            // На момент чтения targetType еще имеет оригинальный тип до ремаппинга, так что валидно
            def mappedMethod = ViewsRemapper.mapMethod(readingContext.viewMethodMeta.targetType, rawMethod)

            // Добавляем метод к результату
            readingContext.result << new MethodInvokeMapper(mappedMethod)
        }
    }
}
