package io.tswf.groovy.bettergroovy.asm.view.reading.mappers

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.asm.view.annotation.MethodInvoke
import io.tswf.groovy.bettergroovy.asm.view.annotation.unsafe.SuperMethodInvoke
import io.tswf.groovy.bettergroovy.asm.view.exception.InvalidViewMethodException
import io.tswf.groovy.bettergroovy.asm.view.generation.mapper.MethodInvokeMapper
import io.tswf.groovy.bettergroovy.asm.view.generation.mapper.SuperMethodInvokeMapper
import io.tswf.groovy.bettergroovy.asm.view.reading.IViewMethodMapperReader
import io.tswf.groovy.bettergroovy.asm.view.reading.ViewMethodMapperReadingContext
import io.tswf.groovy.bettergroovy.asm.view.util.MethodToInvokeResolveHelper
import org.objectweb.asm.commons.Method

@AutoService(IViewMethodMapperReader)
@CompileStatic
class SuperMethodInvokeMapperReader implements IViewMethodMapperReader {

    public static final Collection<String> PREFIXES = MethodInvokeMapperReader.PREFIXES.map { "super${it.with { length() == 1 ? charAt(0).toUpperCase() : "${charAt(0).toUpperCase()}${substring(1)}" }}".toString() }
    public static final String METHOD_NAME_PARAMETER_NAME = "name"
    public static final String METHOD_DESC_PARAMETER_NAME = "desc"

    @Override
    void handleViewMethodReading(ViewMethodMapperReadingContext readingContext) {
        def viewMethod = readingContext.viewMethodMeta.viewMethod
        def methodInvokeAnnotation = viewMethod.findAnnotationByType(SuperMethodInvoke)
        def isViewMethodMatchesPrefixes = PREFIXES.any { prefix -> viewMethod.name.startsWith(prefix) && viewMethod.name.length() > prefix.length() }
        def methodIsInvocation = methodInvokeAnnotation != null || isViewMethodMatchesPrefixes

        if (methodIsInvocation) {
            def methodName = methodInvokeAnnotation?.findValueByName(METHOD_NAME_PARAMETER_NAME, String) ?: MethodToInvokeResolveHelper.resolveTargetMethodNameFromViewMethod(readingContext.viewMethodMeta, PREFIXES)

            def methodDesc = methodInvokeAnnotation?.findValueByName(METHOD_DESC_PARAMETER_NAME, String) ?: viewMethod.desc

            if (methodName.isNullOrEmpty()) {
                throw new InvalidViewMethodException("extracted target method name is empty or null", readingContext.viewMethodMeta)
            }

            if (methodDesc.isNullOrEmpty()) {
                throw new InvalidViewMethodException("extracted target method descriptor is empty or null", readingContext.viewMethodMeta)
            }


            // Добавляем метод к результату. Тут ремаппинга нет потому, что имя суперкласса можно узнать только при выполнеии. Это тот исключительный случай, когда ремаппинг будет в рантайме
            readingContext.result << new SuperMethodInvokeMapper(new Method(methodName, methodDesc))
        }
    }
}
