package io.tswf.groovy.bettergroovy.asm.view.reading.remap

import groovy.transform.CompileStatic
import org.objectweb.asm.Type
import org.objectweb.asm.commons.Method

import java.util.function.Function

@CompileStatic
class ViewsRemapper {

    @Lazy
    static List<IViewsRemappingStrategy> remappingStrategies = ServiceLoader.load(IViewsRemappingStrategy)
        .toList()
            .sortBy(true) { it.priority }

    static Type mapType(Type anotherType) {
        return delegateToStrategies {it.mapType(anotherType)  } ?: anotherType
    }

    static Method mapMethod(Type anotherMethodOwner, Method anotherMethod) {
        return delegateToStrategies {it.mapMethod(anotherMethodOwner, anotherMethod)  } ?: anotherMethod
    }

    static Tuple2<String, Type> mapField(Type anotherFieldOwner, String anotherFieldName, Type anotherFieldType) {
        return delegateToStrategies { it.mapField(anotherFieldOwner, anotherFieldName, anotherFieldType) } ?: new Tuple2<>(anotherFieldName, anotherFieldType)
    }

    protected static <T> T delegateToStrategies(Function<IViewsRemappingStrategy, T> delegate) {
        def result = (Type) null

        for (def strategy : remappingStrategies) {
            result = delegate.apply(strategy)
            if (result != null) {
                return result
            }
        }

        return null
    }
}
